import time

from apscheduler.schedulers.background import BlockingScheduler
from SpiderKeeper.app import app, agent
from SpiderKeeper.app.spider.model import Project, JobInstance, SpiderInstance


scheduler = BlockingScheduler()


def sync_projects():
    """
    sync projects
    :return:
    """
    agent.get_project_list()
    app.logger.debug('[sync_projects]')


def sync_job_execution_status_job():
    """
    sync job execution running status
    :return:
    """
    for project in Project.query.all():
        agent.sync_job_status(project)
    app.logger.debug('[sync_job_execution_status]')


def sync_spiders():
    """
    sync spiders
    :return:
    """
    for project in Project.query.all():
        spider_instance_list = agent.get_spider_list(project)
        SpiderInstance.update_spider_instances(project.id, spider_instance_list)
    app.logger.debug('[sync_spiders]')


def run_spider_job(job_instance_id):
    """
    run spider by scheduler
    :param job_instance_id:
    :return:
    """
    try:
        job_instance = JobInstance.query.get(job_instance_id)
        agent.start_spider(job_instance)
        app.logger.info('[run_spider_job][project:%s][spider_name:%s][job_instance_id:%s]' % (
            job_instance.project_id, job_instance.spider_name, job_instance.id))
    except Exception as e:
        app.logger.error('[run_spider_job] ' + str(e))


def reload_runnable_spider_job_execution():
    """
    add periodic job to scheduler
    :return:
    """
    running_job_ids = set([job.id for job in scheduler.get_jobs()])
    app.logger.debug('[running_job_ids] %s' % ','.join(running_job_ids))
    available_job_ids = set()
    # add new job to schedule
    for job_instance in JobInstance.query.filter_by(enabled=0, run_type="periodic").all():
        job_id = "spider_job_%s:%s" % (
            job_instance.id, int(time.mktime(job_instance.date_modified.timetuple()))
        )
        available_job_ids.add(job_id)
        if job_id not in running_job_ids:
            scheduler.add_job(run_spider_job,
                              args=(job_instance.id,),
                              trigger='cron',
                              id=job_id,
                              minute=job_instance.cron_minutes,
                              hour=job_instance.cron_hour,
                              day=job_instance.cron_day_of_month,
                              day_of_week=job_instance.cron_day_of_week,
                              month=job_instance.cron_month,
                              second=0,
                              max_instances=999,
                              misfire_grace_time=60 * 60,
                              coalesce=True)
            app.logger.info(
                '[load_spider_job][project:%s][spider_name:%s][job_instance_id:%s][job_id:%s]' % (
                    job_instance.project_id, job_instance.spider_name, job_instance.id, job_id
                )
            )
    # remove invalid jobs
    for invalid_job_id in filter(lambda job_id: job_id.startswith("spider_job_"),
                                 running_job_ids.difference(available_job_ids)):
        scheduler.remove_job(invalid_job_id)
        app.logger.info('[drop_spider_job][job_id:%s]' % invalid_job_id)


scheduler.add_job(sync_projects, 'interval', seconds=10, id='sys_sync_projects')
scheduler.add_job(sync_job_execution_status_job, 'interval', seconds=5, id='sys_sync_status')
scheduler.add_job(sync_spiders, 'interval', seconds=10, id='sys_sync_spiders')
scheduler.add_job(reload_runnable_spider_job_execution, 'interval', seconds=30,
                  id='sys_reload_job')



#!/usr/bin/env python

from setuptools import setup, find_packages

from SpiderKeeper import __version__, __author__

setup(
    name='SpiderKeeper-2',
    version=__version__,
    description='Admin ui for spider service',
    long_description=
    'Go to https://github.com/DormyMo/SpiderKeeper/ for more information.',
    author=__author__,
    author_email='nogamemorebrain@gmail.com',
    url='https://github.com/DormyMo/SpiderKeeper/',
    license='MIT',
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'aniso8601==1.2.0',
        'APScheduler==3.3.1',
        'click==6.7',
        'demjson==2.2.4',
        'Flask==0.12.1',
        'Flask-BasicAuth==0.2.0',
        'Flask-RESTful==0.3.5',
        'flask-restful-swagger==0.19',
        'Flask-SQLAlchemy==2.2',
        'itsdangerous==0.24',
        'Jinja2==2.9.6',
        'MarkupSafe==1.0',
        'PyMySQL==0.7.11',
        'python-dateutil==2.6.0',
        'pytz==2017.2',
        'requests==2.13.0',
        'six==1.10.0',
        'SQLAlchemy==1.1.9',
        'tzlocal==1.3',
        'Werkzeug==0.12.1'
    ],

    entry_points={
        'console_scripts': {
            'spiderkeeper = SpiderKeeper.run:main'
        },
    },

    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
    ],
)
